@@ -10,11 +10,13 @@
 
 #include "soup-address.h"
 #include "soup-auth.h"
+#include "soup-connection.h"
 #include "soup-enum-types.h"
 #include "soup-marshal.h"
 #include "soup-message.h"
 #include "soup-message-private.h"
 #include "soup-misc.h"
+#include "soup-socket.h"
 #include "soup-uri.h"
 
 /**
@@ -1401,12 +1403,6 @@ soup_message_cleanup_response (SoupMessage *req)
 	}
 	priv->http_version = priv->orig_http_version;
 
-	if (priv->tls_certificate) {
-		g_object_unref (priv->tls_certificate);
-		priv->tls_certificate = NULL;
-	}
-	priv->tls_errors = 0;
-
 	g_object_notify (G_OBJECT (req), SOUP_MESSAGE_STATUS_CODE);
 	g_object_notify (G_OBJECT (req), SOUP_MESSAGE_REASON_PHRASE);
 	g_object_notify (G_OBJECT (req), SOUP_MESSAGE_HTTP_VERSION);
@@ -1879,6 +1875,34 @@ soup_message_set_first_party (SoupMessage *msg,
 	g_object_notify (G_OBJECT (msg), SOUP_MESSAGE_FIRST_PARTY);
 }
 
+void
+soup_message_set_https_status (SoupMessage *msg, SoupConnection *conn)
+{
+	SoupSocket *sock;
+
+	sock = conn ? soup_connection_get_socket (conn) : NULL;
+	if (sock && soup_socket_is_ssl (sock)) {
+		GTlsCertificate *certificate;
+		GTlsCertificateFlags errors;
+
+		g_object_get (sock,
+			      SOUP_SOCKET_TLS_CERTIFICATE, &certificate,
+			      SOUP_SOCKET_TLS_ERRORS, &errors,
+			      NULL);
+		g_object_set (msg,
+			      SOUP_MESSAGE_TLS_CERTIFICATE, certificate,
+			      SOUP_MESSAGE_TLS_ERRORS, errors,
+			      NULL);
+		if (certificate)
+			g_object_unref (certificate);
+	} else {
+		g_object_set (msg,
+			      SOUP_MESSAGE_TLS_CERTIFICATE, NULL,
+			      SOUP_MESSAGE_TLS_ERRORS, 0,
+			      NULL);
+	}
+}
+
 /**
  * soup_message_get_https_status:
  * @msg: a #SoupMessage
