@@ -21,7 +21,6 @@
 #include "soup-marshal.h"
 #include "soup-misc.h"
 #include "soup-misc-private.h"
-#include "soup-ssl.h"
 
 /**
  * SECTION:soup-socket
@@ -75,11 +74,11 @@ typedef struct {
 
 	guint non_blocking:1;
 	guint is_server:1;
+	guint ssl:1;
 	guint ssl_strict:1;
 	guint ssl_fallback:1;
 	guint clean_dispose:1;
 	gpointer ssl_creds;
-	gboolean ssl_ca_in_creds;
 
 	GMainContext   *async_context;
 	GSource        *watch_src;
@@ -346,6 +345,10 @@ soup_socket_class_init (SoupSocketClass *socket_class)
 	 * Alias for the #SoupSocket:ssl-creds property.
 	 * (SSL credential information.)
 	 **/
+	/* For historical reasons, there's only a single property
+	 * here, which is a GTlsDatabase for client sockets, and
+	 * a GTlsCertificate for server sockets. Whee!
+	 */
 	g_object_class_install_property (
 		object_class, PROP_SSL_CREDENTIALS,
 		g_param_spec_pointer (SOUP_SOCKET_SSL_CREDENTIALS,
@@ -792,6 +795,7 @@ listen_watch (GObject *pollable, gpointer data)
 		new_priv->async_context = g_main_context_ref (priv->async_context);
 	new_priv->non_blocking = priv->non_blocking;
 	new_priv->is_server = TRUE;
+	new_priv->ssl = priv->ssl;
 	if (priv->ssl_creds)
 		new_priv->ssl_creds = priv->ssl_creds;
 	finish_socket_setup (new_priv);
@@ -882,8 +886,6 @@ soup_socket_peer_certificate_changed (GObject *conn, GParamSpec *pspec,
 	SoupSocketPrivate *priv = SOUP_SOCKET_GET_PRIVATE (sock);
 
 	priv->tls_errors = g_tls_connection_get_peer_certificate_errors (G_TLS_CONNECTION (priv->conn));
-	if (priv->ssl_ca_in_creds)
-		priv->tls_errors &= ~G_TLS_CERTIFICATE_UNKNOWN_CA;
 
 	g_object_notify (sock, "tls-certificate");
 	g_object_notify (sock, "tls-errors");
@@ -893,14 +895,7 @@ static gboolean
 soup_socket_accept_certificate (GTlsConnection *conn, GTlsCertificate *cert,
 				GTlsCertificateFlags errors, gpointer sock)
 {
-	SoupSocketPrivate *priv = SOUP_SOCKET_GET_PRIVATE (sock);
-
-	if (soup_ssl_credentials_verify_certificate (priv->ssl_creds,
-						     cert, errors,
-						     &priv->ssl_ca_in_creds))
-		return TRUE;
-
-	return !priv->ssl_strict;
+	return TRUE;
 }
 
 /**
@@ -940,8 +935,8 @@ soup_socket_start_proxy_ssl (SoupSocket *sock, const char *ssl_host,
 
 	if (G_IS_TLS_CONNECTION (priv->conn))
 		return TRUE;
-	if (!priv->ssl_creds)
-		return FALSE;
+
+	priv->ssl = TRUE;
 
 	if (!priv->is_server) {
 		GTlsClientConnection *conn;
@@ -952,7 +947,7 @@ soup_socket_start_proxy_ssl (SoupSocket *sock, const char *ssl_host,
 				       NULL, NULL,
 				       "base-io-stream", priv->conn,
 				       "server-identity", identity,
-				       "use-system-certdb", FALSE,
+				       "database", priv->ssl_creds,
 				       "require-close-notify", FALSE,
 				       "use-ssl3", priv->ssl_fallback,
 				       NULL);
@@ -964,16 +959,18 @@ soup_socket_start_proxy_ssl (SoupSocket *sock, const char *ssl_host,
 		g_object_unref (priv->conn);
 		priv->conn = G_IO_STREAM (conn);
 
-		g_signal_connect (conn, "accept-certificate",
-				  G_CALLBACK (soup_socket_accept_certificate),
-				  sock);
+		if (!priv->ssl_strict) {
+			g_signal_connect (conn, "accept-certificate",
+					  G_CALLBACK (soup_socket_accept_certificate),
+					  sock);
+		}
 	} else {
 		GTlsServerConnection *conn;
 
 		conn = g_initable_new (g_tls_backend_get_server_connection_type (backend),
 				       NULL, NULL,
 				       "base-io-stream", priv->conn,
-				       "certificate", soup_ssl_credentials_get_certificate (priv->ssl_creds),
+				       "certificate", priv->ssl_creds,
 				       "use-system-certdb", FALSE,
 				       "require-close-notify", FALSE,
 				       NULL);
@@ -984,7 +981,6 @@ soup_socket_start_proxy_ssl (SoupSocket *sock, const char *ssl_host,
 		priv->conn = G_IO_STREAM (conn);
 	}
 
-	priv->ssl_ca_in_creds = FALSE;
 	g_signal_connect (priv->conn, "notify::peer-certificate",
 			  G_CALLBACK (soup_socket_peer_certificate_changed), sock);
 
@@ -1000,6 +996,7 @@ soup_socket_handshake_sync (SoupSocket    *sock,
 	SoupSocketPrivate *priv = SOUP_SOCKET_GET_PRIVATE (sock);
 	GError *error = NULL;
 
+	priv->ssl = TRUE;
 	if (g_tls_connection_handshake (G_TLS_CONNECTION (priv->conn),
 					cancellable, &error))
 		return SOUP_STATUS_OK;
@@ -1048,6 +1045,8 @@ soup_socket_handshake_async (SoupSocket         *sock,
 	SoupSocketPrivate *priv = SOUP_SOCKET_GET_PRIVATE (sock);
 	SoupSocketAsyncConnectData *data;
 
+	priv->ssl = TRUE;
+
 	data = g_slice_new (SoupSocketAsyncConnectData);
 	data->sock = g_object_ref (sock);
 	data->callback = callback;
@@ -1065,9 +1064,7 @@ soup_socket_handshake_async (SoupSocket         *sock,
  * soup_socket_is_ssl:
  * @sock: a #SoupSocket
  *
- * Tests if @sock is set up to do SSL. Note that this simply means
- * that the %SOUP_SOCKET_SSL_CREDENTIALS property has been set; it
- * does not mean that soup_socket_start_ssl() has been called.
+ * Tests if @sock is doing (or has attempted to do) SSL.
  *
  * Return value: %TRUE if @sock has SSL credentials set
  **/
@@ -1076,7 +1073,7 @@ soup_socket_is_ssl (SoupSocket *sock)
 {
 	SoupSocketPrivate *priv = SOUP_SOCKET_GET_PRIVATE (sock);
 
-	return priv->ssl_creds != NULL;
+	return priv->ssl;
 }
 
 /**
@@ -1270,9 +1267,6 @@ read_from_network (SoupSocket *sock, gpointer buffer, gsize len,
 							  cancellable);
 		}
 		return SOUP_SOCKET_WOULD_BLOCK;
-	} else if (g_error_matches (my_err, G_TLS_ERROR, G_TLS_ERROR_HANDSHAKE)) {
-		my_err->domain = SOUP_SSL_ERROR;
-		my_err->code = SOUP_SSL_ERROR_CERTIFICATE;
 	}
 
 	g_propagate_error (error, my_err);
@@ -1536,9 +1530,6 @@ soup_socket_write (SoupSocket *sock, gconstpointer buffer,
 						  G_IO_OUT,
 						  socket_write_watch, sock, cancellable);
 		return SOUP_SOCKET_WOULD_BLOCK;
-	} else if (g_error_matches (my_err, G_TLS_ERROR, G_TLS_ERROR_HANDSHAKE)) {
-		my_err->domain = SOUP_SSL_ERROR;
-		my_err->code = SOUP_SSL_ERROR_CERTIFICATE;
 	}
 
 	g_mutex_unlock (priv->iolock);
